local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicKeep.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/simpleRoofs/roofKeep.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/domeRoofs/domeKeep.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/ramparts/rampartKeep.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Keep materials
StoneGate:registerAssetId("models/roofs/simpleRoofs/roofKeep.fbx/Materials/redRoof", StoneGateSetPrefix .. "_KEEP_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/roofs/domeRoofs/domeKeep.fbx/Materials/redRoof", StoneGateSetPrefix .. "_KEEP_DOME_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicKeep.fbx/Materials/murChateau", StoneGateSetPrefix .. "_KEEP_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartKeep.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_RAMPART_KEEP_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/basic/basicKeep.fbx/Materials/murChateauBasement", StoneGateSetPrefix .. "_KEEP_STONE_MANOR_BASEMENT_MATERIAL")
StoneGate:registerAssetId("models/basic/basicKeep.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_KEEP_WHITE_STONE_FLOOR_GROUND_MATERIAL")

-- Register Keep prefab nodes
StoneGate:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_3_Keep_TopPart", StoneGateSetPrefix .. "_KEEP_TOP_PREFAB")

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

registerDefaultBuildingPart(StoneGateSetPrefix .. "_KEEP_TOP")

function registerKeep(_option)
-- Register Keep prefab nodes
    StoneGate:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_0_Keep_Root" .. _option .. "Part", StoneGateSetPrefix .. "_KEEP_ROOT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_1_Keep_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_KEEP_TILING1" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_2_Keep_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_KEEP_TILING2" .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_KEEP_TILING1" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_KEEP_TILING2" .. _option)

-- Register the stone gate set keep root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_KEEP_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_KEEP_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_KEEP_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	BuildingZone = _KeepBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = StoneGateSetPrefix .. "_KEEP_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_KEEP_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_KEEP_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_KEEP_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 8,
	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, "KEEP_ROOT" .. _option)
    })


    StoneGate:registerPrefabComponent("models/basic/basicKeep.fbx/Prefab/keep/step_0_Keep_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
    })
end

registerKeep("") -- with stone roof
registerKeep("P") -- with wooden roof
registerKeep("HALFWHITE") -- Half white with stone ground

function registerRoofPart(_nameFbxFile, _nameFbx, _nameAsset, _woodenCost, _buildingPartType, _angle)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. "/Prefab/keep/Keep_" .. _nameFbx .. _angle .. "Part", StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. _angle .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. _angle .. "_PART",
	Name = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. _angle .. "_NAME",
	Description = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. _angle .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. _angle .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = _woodenCost*_NoSandbox }
		}
	}
    })
    StoneGate:registerPrefabComponent("models/" .. _nameFbxFile .. "/Prefab/keep/Keep_" .. _nameFbx .. _angle .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

function registerRoofPart2(_nameFbxFile, _nameFbx, _nameAsset, _woodenCost, _buildingPartType, _angle)
    registerRoofPart(_nameFbxFile, _nameFbx, _nameAsset, _woodenCost, _buildingPartType, _angle)  
    for i=0, 1 do
        StoneGate:registerPrefabComponent("models/" .. _nameFbxFile .. "/Prefab/keep/Keep_" .. _nameFbx .. _angle .. "Part/Attach_StoneGatePinion_" .. _nameFbx .. i, {
	    DataType = "COMP_BUILDING_ATTACH_NODE",
	    AttachNodeTypeList = { _optionPinionNodeType },
        })
    end
end

function registerRoofShedAngle(_angle)
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof3Shed",                      "ROOF_3SHED",                         5, "MAJOR", _angle )
    registerRoofPart2(  "roofs/simpleRoofs/roofKeep.fbx", "RoofShedExtensionStraightRight", "ROOF_SHED_EXTENSION_STRAIGHT_RIGHT", 1, _optionPinionNodeType, _angle )
    registerRoofPart2(  "roofs/simpleRoofs/roofKeep.fbx", "RoofShedExtensionStraightLeft",  "ROOF_SHED_EXTENSION_STRAIGHT_LEFT",  1, _optionPinionNodeType, _angle )
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof2Shed",                      "ROOF_2SHED",                         5, "MAJOR", _angle )
end
function registerRoofAngle(_angle)
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof3Opened",                    "ROOF_3OPENED",                       5, "MAJOR", _angle )
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof2Opposite",                  "ROOF_2OPPOSITE",                     5, "MAJOR", _angle )
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof4Closed",                    "ROOF_4CLOSED",                       5, "MAJOR", _angle )
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof1Opened",                    "ROOF_1OPENED",                       5, "MAJOR", _angle )
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof2Adjacent",                  "ROOF_2ADJACENT",                     5, "MAJOR", _angle )
    registerRoofPart(  "roofs/simpleRoofs/roofKeep.fbx", "Roof4Opened",                    "ROOF_4OPENED",                       5, "MAJOR", _angle )
    registerRoofPart2( "roofs/simpleRoofs/roofKeep.fbx", "RoofExtensionAdvanced",          "ROOF_EXTENSION_ADVANCED",            1, _optionPinionNodeType, _angle )
    registerRoofPart2( "roofs/simpleRoofs/roofKeep.fbx", "RoofExtensionStraight",          "ROOF_EXTENSION_STRAIGHT",            1, _optionPinionNodeType, _angle )
    registerRoofShedAngle(_angle)
end
registerRoofAngle("45")
registerRoofAngle("30")
registerRoofAngle("60")
registerRoofShedAngle("24")

registerRoofPart(  "roofs/domeRoofs/domeKeep.fbx", "RoofDome2Opposite", "ROOF_DOME_2OPPOSITE", 5, "MAJOR", "" )
registerRoofPart(  "roofs/domeRoofs/domeKeep.fbx", "RoofDome4Closed",   "ROOF_DOME_4CLOSED",   5, "MAJOR", "" )
registerRoofPart(  "roofs/domeRoofs/domeKeep.fbx", "RoofDome1Opened",   "ROOF_DOME_1OPENED",   5, "MAJOR", "" )
registerRoofPart(  "roofs/domeRoofs/domeKeep.fbx", "RoofDome2Adjacent", "ROOF_DOME_2ADJACENT", 5, "MAJOR", "" )
registerRoofPart(  "roofs/domeRoofs/domeKeep.fbx", "RoofDome3Opened",   "ROOF_DOME_3OPENED",   5, "MAJOR", "" )
registerRoofPart(  "roofs/domeRoofs/domeKeep.fbx", "RoofDome4Opened",   "ROOF_DOME_4OPENED",   5, "MAJOR", "" )

---- Ramparts...

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGate:registerAssetId("models/ramparts/rampartKeep.fbx/Prefab/keep/step_4_Keep_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	}
    })
end

registerRampartPart( "Rampart3Opened",   "RAMPART_3OPENED" )
registerRampartPart( "Rampart2Opposite", "RAMPART_2OPPOSITE" )
registerRampartPart( "Rampart4Closed",   "RAMPART_4CLOSED" )
registerRampartPart( "Rampart1Opened",   "RAMPART_1OPENED" )
registerRampartPart( "Rampart2Adjacent", "RAMPART_2ADJACENT" )
registerRampartPart( "Rampart4Opened",   "RAMPART_4OPENED" )
