local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicLargeGate.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicBasementGate.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Keep materials
StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Materials/murChateau", StoneGateSetPrefix .. "_LARGE_GATE_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_LARGE_GATE_WHITE_STONE_FLOOR_GROUND_MATERIAL")
StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_LARGE_GATE_WHITE_STONE_FRAME_MATERIAL")

-- Register Large Gate prefab nodes
StoneGate:registerAssetId("models/basic/basicBasementGate.fbx/Prefab/basementGate/step_1_BGate_Tiling1Part", StoneGateSetPrefix .. "_BGATE_TILING1_PREFAB")

-- Register LargeGate prefab nodes
StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_3_LGate_TopPart", StoneGateSetPrefix .. "_LGATE_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList = {
        StoneGateSetPrefix .. "_LGATE_PILLAR1",
        StoneGateSetPrefix .. "_LGATE_PILLAR2",
        StoneGateSetPrefix .. "_LGATE_PILLAR1P",
        StoneGateSetPrefix .. "_LGATE_PILLAR2P",
	StoneGateSetPrefix .. "_LGATE_TOP",
	StoneGateSetPrefix .. "_LGATE_TILING1",
	StoneGateSetPrefix .. "_LGATE_TILING2",
	StoneGateSetPrefix .. "_LGATE_TILING1P",
	StoneGateSetPrefix .. "_LGATE_TILING2P",
        StoneGateSetPrefix .. "_LGATE_2_PILLAR1",
        StoneGateSetPrefix .. "_LGATE_2_PILLAR2",
        StoneGateSetPrefix .. "_LGATE_2_PILLAR1P",
        StoneGateSetPrefix .. "_LGATE_2_PILLAR2P",
	StoneGateSetPrefix .. "_LGATE_TILING1_2",
	StoneGateSetPrefix .. "_LGATE_TILING1_2P",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR1",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR2",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR3",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR1P",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR2P",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR3P",
	StoneGateSetPrefix .. "_LGATE_TILING1_3",
	StoneGateSetPrefix .. "_LGATE_TILING1_3P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR1",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR2",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR3",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR4",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR1P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR2P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR3P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR4P",
	StoneGateSetPrefix .. "_LGATE_TILING1_4",
	StoneGateSetPrefix .. "_LGATE_TILING1_4P",
	StoneGateSetPrefix .. "_BGATE_TILING1"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerLargeGate2(_option)
    -- Register LargeGate prefab nodes
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_2_LGate_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PREFAB")

    function registerLargeGate2Type(_type, _buildingZone)
        -- Register LargeGate prefab nodes
        StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate" .. _type .. "_Root" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE" .. _type .. "_ROOT" .. _option .. "_PREFAB")
        StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate" .. _type .. "_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate" .. _type .. "_Part", StoneGateSetPrefix .. "_LGATE" .. _type .. "_PILLAR1" .. _option .. "_PREFAB")
        StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate" .. _type .. "_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate" .. _type .. "_Part", StoneGateSetPrefix .. "_LGATE" .. _type .. "_PILLAR2" .. _option .. "_PREFAB")
        StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_1_LGate_Tiling1" .. _type .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING1" .. _type .. _option .. "_PREFAB")

        -- Register the stone gate set main root part
        StoneGate:register({
	    DataType = "BUILDING_PART",
	    Id = StoneGateSetPrefix .. "_LGATE" .. _type .. "_ROOT" .. _option .. "_PART",
	    Name = StoneGateSetPrefix .. "_LGATE" .. _type .. "_ROOT" .. _option .. "_NAME",
	    Description = StoneGateSetPrefix .. "_LGATE" .. _type .. "_ROOT" .. _option .. "_DESC",
	    Category = "CORE",
            BuildingZone = _buildingZone,
	    ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_LGATE" .. _type .. "_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_LGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING1" .. _type .. _option .. "_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
            },
	    Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	    },
	    EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	    }
        })

        -- Register grass clearing components
        StoneGate:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate" .. _type .. "_Root" .. _option .. "Part", {
	    DataType = "COMP_DIRT_RECTANGLE",
	    Size = { 8, 8 }
        })
    end

    registerLargeGate2Type("", _LGate2BuildingZone)     -- 2 opposite opened sides
    registerLargeGate2Type("_2", _LGate2_2BuildingZone) -- 2 adjacent opened sides
end

registerLargeGate2("") -- with stone ground
registerLargeGate2("P") -- with wooden ground

function registerLargeGate3(_option)
    -- Register LargeGate prefab nodes
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_3_Part", StoneGateSetPrefix .. "_LGATE_3_PILLAR1" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_3_Part", StoneGateSetPrefix .. "_LGATE_3_PILLAR2" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar3" .. _option .. "_LGate_3_Part", StoneGateSetPrefix .. "_LGATE_3_PILLAR3" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_1_LGate_Tiling1_3" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING1_3" .. _option .. "_PREFAB")

    -- Register the stone gate set main root 3 accesses part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _LGate3BuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_LGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING1_3" .. _option .. "_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
    })
end

registerLargeGate3("") -- with stone ground
registerLargeGate3("P") -- with wooden ground

function registerLargeGate4(_option)
    -- Register LargeGate prefab nodes
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR1" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR2" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar3" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR3" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar4" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR4" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_1_LGate_Tiling1_4" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING1_4" .. _option .. "_PREFAB")

    -- Register the stone gate set main root 4 accesses part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _LGate4BuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_LGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING1_4" .. _option .. "_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
    })
end

registerLargeGate4("") -- with stone ground
registerLargeGate4("P") -- with wooden ground
