local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/decorations/decoration.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/decorations/pinion.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Decorations materials
StoneGate:registerAssetId("models/decorations/decoration.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_DECORATIONS_DECORATION_WHITE_STONE_FRAME_MATERIAL")
StoneGate:registerAssetId("models/decorations/pinion.fbx/Materials/murChateau", StoneGateSetPrefix .. "_DECORATIONS_PINION_STONE_MANOR_WALL_MATERIAL")

-- Create default minor decoration part registering function 
function registerDefaultMinorDecorationPart(_nameFbxFile, _nameFbx, _nameAsset, _stoneCost, _buildingPartType)
    StoneGate:registerAssetId("models/decorations/" .. _nameFbxFile .. ".fbx/Prefab/decoration/Minor/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_DECORATION_" .. _nameAsset .. "_PREFAB")

    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DECORATION_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_DECORATION_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_DECORATION_" .. _nameAsset .. "_DESC",
	Category = "DECORATION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DECORATION_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _stoneCost*_NoSandbox }
		}}
	},
        IsVisibleWhenBuilt = ttrue
    })

    StoneGate:registerPrefabComponent("models/decorations/" .. _nameFbxFile .. ".fbx/Prefab/decoration/Minor/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

registerDefaultMinorDecorationPart("decoration", "DecorationDoorA", "DOOR_A", 1, "MINOR")
registerDefaultMinorDecorationPart("decoration", "DecorationDoorB", "DOOR_B", 1, "MINOR")
registerDefaultMinorDecorationPart("decoration", "DecorationDoorMA", "DOOR_MA", 1, "MINOR")
registerDefaultMinorDecorationPart("decoration", "DecorationDoorMB", "DOOR_MB", 1, "MINOR")

function registerDefaultMinorDecorationPart2(_nameFbxFile, _nameFbx, _nameAsset, _stoneCost)
    registerDefaultMinorDecorationPart(_nameFbxFile, _nameFbx, _nameAsset, _stoneCost, _optionPinionNodeType)
    for i=0, 1 do
        StoneGate:registerPrefabComponent("models/decorations/" .. _nameFbxFile .. ".fbx/Prefab/decoration/Minor/" .. _nameFbx .. "Part/Attach_StoneGatePinion_" .. _nameFbx .. i, {
	    DataType = "COMP_BUILDING_ATTACH_NODE",
	    AttachNodeTypeList = { _optionPinionNodeType },
        })
    end
end

registerDefaultMinorDecorationPart("pinion", "Pinion2Stone45", "PINION2_STONE45", 2, "MINOR")

function registerPinionColor(_color, _COLOR, _stoneCost)
    function registerPinion2ShedAngle(_angle)
        registerDefaultMinorDecorationPart2("pinion", "Pinion2LeftShed" .. _color .. _angle, "PINION2_LEFT_SHED_" .. _COLOR .. _angle, _stoneCost)
        registerDefaultMinorDecorationPart2("pinion", "Pinion2RightShed" .. _color .. _angle, "PINION2_RIGHT_SHED_" .. _COLOR .. _angle, _stoneCost)
    end
    function registerPinionAngle(_angle)
        registerDefaultMinorDecorationPart("pinion", "Pinion1Simple" .. _color .. _angle, "PINION1_SIMPLE_" .. _COLOR .. _angle, _stoneCost, "MINOR")
        registerDefaultMinorDecorationPart("pinion", "Pinion1Shed" .. _color .. _angle, "PINION1_SHED_" .. _COLOR .. _angle, _stoneCost, "MINOR")
        registerDefaultMinorDecorationPart2("pinion", "Pinion2Simple" .. _color .. _angle, "PINION2_SIMPLE_" .. _COLOR .. _angle, _stoneCost)
        registerPinion2ShedAngle(_angle)
    end
    registerPinionAngle("45")
    registerPinionAngle("30")
    registerPinion2ShedAngle("24")
    registerPinionAngle("60")
end
registerPinionColor("Cream", "CREAM", 0)
registerPinionColor("White", "WHITE", 0)
registerPinionColor("Stone", "STONE", 2)
registerPinionColor("Wood",  "WOOD",  0)


